/*
 * spi.c
 *
 *  Created on: Nov 6, 2022
 *      Author: hussamaldean
 */


#include <SPI1.h>
#include "stm32f4xx.h"


void SPI1_Pins_Init(void)
{
	RCC->AHB1ENR|=RCC_AHB1ENR_GPIOAEN; //enable clock for GPIOA

	GPIOA->MODER|=GPIO_MODER_MODE7_1; // PA7 to alternate function mode
	GPIOA->MODER &=~(GPIO_MODER_MODE7_0);

	GPIOA->OSPEEDR|=GPIO_OSPEEDER_OSPEEDR7;

	GPIOA->AFR[0]|=(0x05<<28);
}



void SPI1_Config(void)
{
	/*Enable clock access to SPI1 module*/
	RCC->APB2ENR |= RCC_APB2ENR_SPI1EN;

	/*Set clock to fPCLK/2*/
	SPI1->CR1 &=~SPI_CR1_BR_0;
	SPI1->CR1 &=~SPI_CR1_BR_1;
	SPI1->CR1 |=SPI_CR1_BR_2;

	/*Set CPOL to 0 and CPHA to 0 (MODE0)*/
	SPI1->CR1 &=~(SPI_CR1_CPOL);
	SPI1->CR1 &=~(SPI_CR1_CPHA);

	/*Enable full duplex*/
	SPI1->CR1 &=~(SPI_CR1_RXONLY);

	/*Set MSB first*/
	SPI1->CR1 &= ~(SPI_CR1_LSBFIRST);

	/*Set mode to MASTER*/
	SPI1->CR1 |= (SPI_CR1_MSTR);

	/*Set 8 bit data mode*/
	SPI1->CR1 &= ~(SPI_CR1_DFF);

	/*Select software slave management by
	* setting SSM=1 and SSI=1*/
	SPI1->CR1 |= (SPI_CR1_SSM);
	SPI1->CR1 |= (SPI_CR1_SSI);

	/*Enable SPI module*/
	SPI1->CR1 |= (SPI_CR1_SPE);
}

void SPI1_Transmit(uint8_t *data,uint32_t size)
{
	uint32_t i=0;

	while(i<size)
	{
		/*Wait until TXE is set*/
		while(!(SPI1->SR & (SPI_SR_TXE))){}

		/*Write the data to the data register*/
		SPI1->DR = data[i];
		i++;
	}
	/*Wait until TXE is set*/
	while(!(SPI1->SR & (SPI_SR_TXE))){}

	/*Wait for BUSY flag to reset*/
	while((SPI1->SR & (SPI_SR_BSY))){}

	/*Clear OVR flag*/
	(void)SPI1->DR;
	(void)SPI1->SR;
}
